@@ -9,23 +9,48 @@ from .utils.ut import Utils
 from .utils.ut_autprog import AutProgUtils
 
 class Schedule():
-    def __init__(self, accessToken:str, endpoint:str) -> None:
-        #endpoint = 'http://172.16.2.206:8130/graphql/'
-        
+    def __init__(self, accessToken:str, endpoint:str, client:object) -> None:
+              
         header = {
             'authorization': 'Bearer ' + Automation.accessToken
         }
         
-        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=True)
+        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=True, proxies=client.proxies)
         self.client = Client(transport=transport, fetch_schema_from_transport=False)
+        self.raiseException = client.raiseException
+        self.defaults = client.defaults
 
-    def schedules(self):
-        "Lists schedules of all workflows"
+    def schedules(
+        self, 
+        fields:list=None, 
+        where:str=None) -> pd.DataFrame:
+        """
+        Returns schedules in a DataDrame
 
-        graphQLString = f'''query schedules {{
-            schedules     
-                {{
-                scheduleId
+        Parameters:
+        -----------
+        fields: list | str = None
+            A list of all properties to be queried. If None, all properties will be queried.
+        where: str = None
+            Use a string to add where criteria like
+            ''workflowId eq "meteoData"'.
+
+        Example:
+        --------
+        >>> Schedule.schedules(
+                where='workflowId == "meteoData"', 
+                fields=['name', 'cron']
+            )   
+        """
+
+        key = 'schedules'
+
+        if fields != None:
+            if type(fields) != list:
+                fields = [fields]
+            _fields = Utils._queryFields(fields, recursive=True)   
+        else:
+            _fields =f'''scheduleId
                 name
                 description
                 workflowId
@@ -33,6 +58,18 @@ class Schedule():
                 cron
                 isActive
                 nextFireTime
+                variables {{
+                    key
+                    value
+                }}''' 
+
+        resolvedFilter = ''
+        if where != None: 
+            resolvedFilter = f'({Utils._resolveWhereString(where)})'
+
+        graphQLString = f'''query schedules {{
+            {key}{resolvedFilter}  {{
+                {_fields}
             }}
         }}
         '''
@@ -40,7 +77,7 @@ class Schedule():
         result = Utils._executeGraphQL(self, graphQLString)
         if result == None: return
 
-        df = pd.json_normalize(result['schedules'])
+        df = pd.json_normalize(result[key])
         return df
 
     def createSchedule(self, name:str, workflowId:str, businessKey:str, cron:str, 
@@ -200,7 +237,6 @@ class Schedule():
                 logger.info(f"Schedule {scheduleId} deleted")
                 return None
 
-
     def nextFireTimes(self, workflowId:str, fromTimepoint:str=None, toTimepoint:str=None, count:int=None):
         """Show next fire times of a workflow"""
 
@@ -239,7 +275,7 @@ class Schedule():
 class Automation():
     accessToken = ''
 
-    def __init__(self, accessToken:str, endpoint:str) -> None:
+    def __init__(self, accessToken:str, endpoint:str, client:object) -> None:
         
         Automation.accessToken = accessToken
         
@@ -247,8 +283,10 @@ class Automation():
             'authorization': 'Bearer ' + accessToken
         }
         
-        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=False)
+        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=False, proxies=client.proxies)
         self.client = Client(transport=transport, fetch_schema_from_transport=True)
+        self.raiseException = client.raiseException
+        self.defaults = client.defaults
 
     def workflows(self) -> pd.DataFrame:
         """Returns a DataFrame of all Workflows"""
@@ -443,7 +481,7 @@ class Automation():
 
         return result
 
-    def deleteWorkflow(self, workflowId):
+    def deleteWorkflow(self, workflowId:str):
         """Deletes a workflow"""
 
         graphQLString = f'''mutation deleteWorkflow {{
@@ -465,8 +503,6 @@ class Automation():
         '''
 
         result = Utils._executeGraphQL(self, graphQLString)
-        if result == None: return
-
         return result
 
     def terminateWorkflowInstance(self, workflowInstanceId):
@@ -490,8 +526,58 @@ class Automation():
         '''
 
         result = Utils._executeGraphQL(self, graphQLString)
-        if result == None: return
-
         return result
 
+    def updateWorkflow(self, workflowId:str, name:str=None, description:str=None):
+        """Updates a workflow (name and description can be changed)"""
 
+        name = Utils._argNone('name', name)
+        description = Utils._argNone('description', description)
+
+        key = 'updateWorkflow'
+        graphQLString = f'''mutation updateWorkflow {{
+            {key}(workflowId: "{workflowId}", properties: {{
+                {description}
+                {name}
+                }}) {{
+                    ... on UpdateWorkflowError {{
+                    message
+                    }}
+                    ... on WorkflowNotFound {{
+                    workflowId
+                    message
+                    }}
+                    ... on WorkflowUpdated {{
+                    workflow {{
+                        id
+                        name
+                        description
+                        }}
+                    }}
+                }}
+            }}
+            '''
+
+        result = Utils._executeGraphQL(self, graphQLString)
+        return result
+
+    def retryTask(self, externalTaskId):
+        key = 'retryTask'
+
+        graphQLString = f'''mutation retryTask {{
+            {key}(externalTaskId: "{externalTaskId}") {{
+                ... on RetryTaskError {{
+                message
+                }}
+                ...on TaskNotFound {{
+                message
+                }}
+                ... on TaskRetried {{
+                success
+                }}
+            }}
+        }}
+        '''
+
+        result = Utils._executeGraphQL(self, graphQLString)
+        return result
