@@ -1,9 +1,13 @@
 import pandas
+import pytz
+import csv
+
 from re import match
 from loguru import logger
 from tzlocal import get_localzone
 from datetime import datetime
-import pytz
+from pathlib import Path
+
 
 class UtilsProperty():
     def _getDataTypes(properties:dict) -> dict:
@@ -14,6 +18,9 @@ class UtilsProperty():
 
         logger.debug(properties)
         propertyTypes = {'Boolean': [], 'Numeric': [], 'DateTime': [], 'String': [], 'Unknown': []}
+        if 'dataType' not in properties[0]:
+            return propertyTypes
+
         for property in properties:
             if property['dataType'] in ['DATE_TIME', 'DATE_TIME_OFFSET']:
                 propertyTypes['DateTime'].append(property['name'])
@@ -40,6 +47,12 @@ class UtilsProperty():
         p = [property['name'] for property in properties if property['nullable'] == True]
         return p
 
+    def _getReferences(properties:dict) -> list:
+        """ Lists properties that are nullable """
+
+        p = [property['name'] for property in properties if property['type'] == 'reference']
+        return p
+
     def _transformBool(value:str):
         trueValues = ['ja', 'Ja', 'yes', 'Yes', 'True', 'true', 1]
         falseValues = ['nein', 'Nein', 'no', 'No', 'false','False',0]
@@ -74,7 +87,8 @@ class UtilsProperty():
     def _transformList(pythonList:str, propertyType:str) -> list:
         """ Transforms a String into a Python list"""
         if not UtilsProperty._checkIsList(pythonList):
-            logger.warning(f"Value '{pythonList}' is not array type!")
+            #logger.warning(f"Value '{pythonList}' is not array type, try to transform...")
+            pythonList = '[' + pythonList + ']'
 
         pythonList = pythonList[1:-1].split(',')
         pythonList = [element.strip() for element in pythonList]
@@ -92,51 +106,80 @@ class UtilsProperty():
 
 class FileUtils(UtilsProperty):
 
-    def _createItems(content:list, dataType:dict, isArray:list, nullable:list):
-        itemList = []
-        header = content[0]
-        for i, row in enumerate(content):
-            itemOkay = True # is changed to false, if certain criteria are not met -> warning message, next item
-            if i == 0: continue
-            if len(row) < 1: continue
+    def _createItems(content:list, dataType:dict, isArray:list, nullable:list, referenceMapping=None, type=None):
+        
+        def _createItem():
+            if i == 0: return
+            if len(row) < 1: return
             else:
-                item = {}
+                if type == None:
+                    item = {}
+                if type == 'timeSeries':
+                    item = {'resolution': {}}
                 for j, field in enumerate(row):
                     if not field:
                         if header[j] in nullable:
                             continue
                         elif header[j] not in nullable and len(field) > 0:
-                            logger.warning(f"Value missing for non nullable {header[j]}. Item from line {i} not imported.")
-                            itemOkay = False
-                            break
+                            logger.warning(f"Line {i}, field '{header[j]}': Value missing for non nullable property.")
+                            return
                         else:
                             pass
                     else:
-                        if header[j] in dataType['Boolean']:
-                            if header[j] in isArray:
-                                item.setdefault(header[j], UtilsProperty._transformList(field, 'Boolean'))
-                            else: 
-                                item.setdefault(header[j], UtilsProperty._transformBool(field))
-                        elif header[j] in dataType['Numeric']:
-                            if header[j] in isArray:
-                                item.setdefault(header[j], UtilsProperty._transformList(field,'Numeric'))
+                        if referenceMapping != None and header[j] in referenceMapping:
+                            parentHeader = header[j].split('.')[0]
+                            if parentHeader in isArray:
+                                nameList = UtilsProperty._transformList(field, 'String')
+                                try:
+                                    idList = [referenceMapping[header[j]][item] for item in nameList]
+                                except KeyError as key:
+                                    logger.warning(f"Line {i}, field '{header[j]}': {key} is not a valid reference.")
+                                    return
+                                item.setdefault(parentHeader, idList)
                             else:
-                                item.setdefault(header[j], UtilsProperty._transformNumeric(field))
-                        elif header[j] in dataType['DateTime']:
-                            if header[j] in isArray:
-                                item.setdefault(header[j], UtilsProperty._transformList(field, 'DateTime'))
-                            else:
-                                #item.setdefault(header[j], FileUtils._convertTimestamp(field, timeZone, FileUtils._dateFormat(field)))
-                                item.setdefault(header[j], field)
+                                try:
+                                    item.setdefault(parentHeader, referenceMapping[header[j]][field])
+                                except KeyError as key:
+                                    logger.warning(f"Line {i}, field '{header[j]}': {key} is not a valid reference.")
+                                    return
                         else:
-                            if header[j] in isArray:
-                                item.setdefault(header[j], UtilsProperty._transformList(field, 'String'))
+                            if header[j] in dataType['Boolean']:
+                                if header[j] in isArray:
+                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'Boolean'))
+                                else: 
+                                    item.setdefault(header[j], UtilsProperty._transformBool(field))
+                            elif header[j] in dataType['Numeric']:
+                                if header[j] in isArray:
+                                    item.setdefault(header[j], UtilsProperty._transformList(field,'Numeric'))
+                                else:
+                                    item.setdefault(header[j], UtilsProperty._transformNumeric(field))
+                            elif header[j] in dataType['DateTime']:
+                                if header[j] in isArray:
+                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'DateTime'))
+                                else:
+                                    #item.setdefault(header[j], FileUtils._convertTimestamp(field, timeZone, FileUtils._dateFormat(field)))
+                                    item.setdefault(header[j], field)
                             else:
-                                item.setdefault(header[j], field)
-                if itemOkay and len(item) > 0:
-                    itemList.append(item)
+                                if type == 'timeSeries' and header[j] == 'timeUnit':
+                                    item['resolution'].setdefault('timeUnit', field)
+                                elif type == 'timeSeries' and header[j] == 'factor':
+                                    item['resolution'].setdefault('factor', field)
+                                elif header[j] in isArray:
+                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'String'))
+                                else:
+                                    item.setdefault(header[j], field)
+                return item
+                    
+        itemList = []
+        header = content[0]
+        for i, row in enumerate(content):
+            item = _createItem()
+            if item != None:           
+                itemList.append(item)
 
-        return itemList
+        return itemList          
+
+ 
 
     def _createInstanceItems(content:list, dataType:dict, isArray:list, nullable:list, idMapping:dict=None, 
         transpose:bool=False) -> list:
@@ -237,7 +280,8 @@ class FileUtils(UtilsProperty):
                             if header[j] in isArray:
                                 item.setdefault(header[j], UtilsProperty._transformList(field, 'DateTime'))
                             else:
-                                item.setdefault(header[j], UtilsProperty._convertTimestamp(field, timeZone))
+                                #item.setdefault(header[j], UtilsProperty._convertTimestamp(field, timeZone))
+                                item.setdefault(header[j], field)
                         else:
                             if header[j] == 'timeUnit':
                                 item['resolution'].setdefault('timeUnit', field)
@@ -253,17 +297,17 @@ class FileUtils(UtilsProperty):
 
         return itemList
 
-    def _comparePropertiesBasic(properties:pandas.DataFrame, header:list) -> None:
+    def _comparePropertiesBasic(properties:list, header:list) -> None:
         """ Compares header with requested properties from a basic inventory. """
 
-        properties = set(properties['name'])
+        properties = set(properties)
         header = set(header)
         return header.difference(properties)
 
-    def _comparePropertiesTimeSeries(properties:pandas.DataFrame, header:list) -> None:
+    def _comparePropertiesTimeSeries(properties:list, header:list) -> None:
         """ Compares header with requested properties from a time series inventory. """
 
-        properties = set(properties['name'])
+        properties = set(properties)
         header = set(header)
         tsProperties = set(['timeUnit', 'factor', 'unit'])
         header = header - tsProperties
@@ -272,14 +316,17 @@ class FileUtils(UtilsProperty):
     def _analyzeProperties(inventoryName:str, properties:pandas.DataFrame) -> tuple:
         """ Analyzes inventory properties and returns dictionaries to each property attribute """
 
-        properties = properties[['name', 'dataType', 'isArray', 'nullable']].to_dict('records')
+        #properties = properties[['name', 'dataType', 'isArray', 'nullable']].to_dict('records')
+        properties = properties.to_dict('records')
         dataType = UtilsProperty._getDataTypes(properties)
         isArray = UtilsProperty._getArrays(properties)
         nullable = UtilsProperty._getNullables(properties)
+        isReference = UtilsProperty._getReferences(properties)
 
         logger.debug(f'Data types: {dataType}')
         logger.debug(f'Array properties: {isArray}')
         logger.debug(f'Nullable properties: {nullable}')
+        logger.debug(f'Reference properties: {isReference}')
 
         return dataType, isArray, nullable
 
@@ -374,3 +421,91 @@ class FileUtils(UtilsProperty):
                 itemContent.append(row)
 
         return itemContent
+
+    def _checkFilePath(filePath, raiseException):
+        """Checks and handles the file path"""
+        filePath = Path(filePath)
+        if str(filePath).lower().endswith('csv'):
+            if not filePath.exists():
+                msg = f"File path {filePath} does not exist"
+                if raiseException: raise Exception(msg)
+                else:
+                    logger.error(msg)
+                    return 
+            return [filePath]
+
+        else:
+            if not filePath.exists():
+                msg = f"File path {filePath} does not exist"
+                if raiseException: raise Exception(msg)
+                else:
+                    logger.error(msg)
+                    return 
+            files = [file for file in filePath.iterdir() if str(file).lower().endswith('csv')]
+            msg = f"No csv files found in {filePath}"
+            if len(files) == 0:
+                if raiseException: raise Exception(msg)
+                else:
+                    logger.error(msg)
+                    return
+            logger.debug(f"FileUtils._checkFilePath: {files}")
+            return files
+
+    def _readCsvFile(file, delimiter) -> list:
+        with open(file) as f:
+            csv_file = csv.reader(f, delimiter=delimiter)
+            content = [row for row in csv_file]
+            logger.debug(f"CSV file '{file.name}' read.")
+        return content
+            
+    def _checkReferences(fields:list) -> list:
+        """
+        Checks, if reference fields have multilevel and returns a list of one-level
+        fields and error an list
+        """
+        references = []
+        errors = []
+        for field in fields:
+            if field.count('.') == 0:
+                continue
+            elif field.count('.') == 1:
+                references.append(field)
+            elif field.count('.') > 1:
+                errors.append(field)
+            else:
+                pass
+
+        return references, errors
+
+    def _createReferenceMapping(client, inventoryName:str, referenceFields:list, isArray:list, content:list, ) -> dict:
+        """
+        Creates a dictionary of mapping inventoryItemIds with the import items of reference.
+        """
+        references = {}
+        content = pandas.DataFrame(content[1:], columns=content[0])
+        for field in referenceFields:
+            parentField = field.split('.')[0]
+            childField = field.split('.')[1]
+            sysId = False
+            if childField == 'sys_inventoryItemId':
+                sysId = True
+            if parentField in isArray:
+                listOfLists = []
+                for item in content[field]:
+                    if len(item) == 0:
+                        continue
+                    listOfLists.append(UtilsProperty._transformList(item, 'String'))
+                nameList = list(set().union(*listOfLists))
+            else:
+                nameList = list(set(content[field]))
+            
+            if sysId == False:
+                inventory = client.structure[inventoryName]['properties'][parentField]['inventoryName']
+                df = client.items(inventory, fields=['sys_inventoryItemId', childField], where=f'{childField} in {nameList}')
+                mapping = {item[1][childField]: item[1]['sys_inventoryItemId'] for item in df.iterrows()}
+            else:
+                mapping = {item: item for item in nameList}
+            
+            references.setdefault(field, mapping)
+
+        return references
\ No newline at end of file
